@@ -1,5 +1,5 @@
 ;; package.lisp -- 
-;; Copyright (C) 2003,2007 Shawn Betts
+;; Copyright (C) 2003-2008 Shawn Betts
 ;;
 ;;  This file is part of stumpwm.
 ;;
@@ -14,9 +14,12 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with this software; see the file COPYING.  If not, write to
-;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
-;; Boston, MA 02111-1307 USA
+;; along with this software; see the file COPYING.  If not, see
+;; <http://www.gnu.org/licenses/>.
 
 (defpackage :stumpwm
-  (:use :cl))
+  (:use :cl)
+  (:shadow #:yes-or-no-p #:y-or-n-p))
+
+(defpackage :stumpwm-user
+  (:use :cl :stumpwm))
