@@ -43,6 +43,7 @@
           *focus-window-hook*
           *place-window-hook*
           *start-hook*
+          *quit-hook*
           *internal-loop-hook*
           *event-processing-hook*
           *focus-frame-hook*
@@ -53,6 +54,8 @@
           *focus-group-hook*
           *key-press-hook*
           *root-click-hook*
+          *new-mode-line-hook*
+          *destroy-mode-line-hook*
           *mode-line-click-hook*
           *display*
           *shell-program*
@@ -118,6 +121,7 @@
           concat
           data-dir-file
           dformat
+          flatten
           define-frame-preference
           redirect-all-output
           remove-hook
@@ -126,9 +130,10 @@
           command-mode-start-message
           command-mode-end-message
           split-string
-	  with-restarts-menu
+          with-restarts-menu
           with-data-file
-	  move-to-head))
+          move-to-head
+          format-expand))
 
 
 ;;; Message Timer
@@ -217,6 +222,9 @@ window group and frame")
 (defvar *start-hook* '()
   "A hook called when stumpwm starts.")
 
+(defvar *quit-hook* '()
+  "A hook called when stumpwm quits.")
+
 (defvar *internal-loop-hook* '()
   "A hook called inside stumpwm's inner loop.")
 
@@ -259,6 +267,14 @@ sequence it is a part of, and command value bound to the key.")
 window. Called with 4 arguments, the screen containing the root
 window, the button clicked, and the x and y of the pointer.")
 
+(defvar *new-mode-line-hook* '()
+  "Called whenever the mode-line is created. It is called with argument,
+the mode-line")
+
+(defvar *destroy-mode-line-hook* '()
+  "Called whenever the mode-line is destroyed. It is called with argument,
+the mode-line")
+
 (defvar *mode-line-click-hook* '()
   "Called whenever the mode-line is clicked. It is called with 4 arguments,
 the mode-line, the button clicked, and the x and y of the pointer.")
@@ -362,6 +378,10 @@ are valid values.
 @item :bottom-left
 @item :bottom-right
 @item :center
+@item :top
+@item :left
+@item :right
+@item :bottom
 @end table")
 
 ;; line editor
@@ -377,6 +397,10 @@ are valid values.
 @item :bottom-left
 @item :bottom-right
 @item :center
+@item :top
+@item :left
+@item :right
+@item :bottom
 @end table")
 
 ;; default values. use the set-* functions to these attributes
@@ -455,7 +479,7 @@ Use the window's resource name.
    (float-unfocus-color :initform nil :accessor screen-float-unfocus-color)
    (msg-border-width :initform nil :accessor screen-msg-border-width)
    (frame-outline-width :initform nil :accessor screen-frame-outline-width)
-   (font :initform nil :accessor screen-font)
+   (fonts :initform '(nil) :accessor screen-fonts)
    (mapped-windows :initform nil :accessor screen-mapped-windows :documentation
     "A list of all mapped windows. These are the raw xlib:window's. window structures are stored in groups.")
    (withdrawn-windows :initform nil :accessor screen-withdrawn-windows :documentation
@@ -490,12 +514,19 @@ exist, in which case they go into the current group.")
    (last-msg-highlights :initform nil :accessor screen-last-msg-highlights)))
 
 (defstruct ccontext
+  screen
   win
   px
   gc
   default-fg
   default-bright
-  default-bg)
+  default-bg
+  fg
+  bg
+  brightp
+  reversep
+  color-stack
+  font)
 
 (defun screen-message-window (screen)
   (ccontext-win (screen-message-cc screen)))
@@ -506,6 +537,9 @@ exist, in which case they go into the current group.")
 (defun screen-message-gc (screen)
   (ccontext-gc (screen-message-cc screen)))
 
+(defun screen-font (screen)
+  (first (screen-fonts screen)))
+
 (defmethod print-object ((object frame) stream)
   (format stream "#S(frame ~d ~a ~d ~d ~d ~d)"
           (frame-number object) (frame-window object) (frame-x object) (frame-y object) (frame-width object) (frame-height object)))
@@ -745,7 +779,7 @@ before reopening.")
 
 (defun redirect-all-output (file)
   "Elect to redirect all output to the specified file. For instance,
-if you want everything to go to ~/stumpwm.d/debug-output.txt you would
+if you want everything to go to ~/.stumpwm.d/debug-output.txt you would
 do:
 
 @example
@@ -762,7 +796,6 @@ do:
 
 ;;; 
 ;;; formatting routines
-
 (defun format-expand (fmt-alist fmt &rest args)
   (let* ((chars (coerce fmt 'list))
          (output "")
@@ -871,13 +904,13 @@ The group's name.
 (defvar *list-hidden-groups* nil
   "Controls whether hidden groups are displayed by 'groups' and 'vgroups' commands")
 
-(defun font-height (font)
-  (+ (xlib:font-descent font)
-     (xlib:font-ascent font)))
+;; (defun font-height (font)
+;;   (+ (font-descent font)
+;;      (font-ascent font)))
 
 (defvar *x-selection* nil
-  "This holds stumpwm's current selection. It is generally set
-when killing text in the input bar.")
+  "This is a plist of stumpwm's current selections. The different properties are
+generally set when killing text in the input bar.")
 
 (defvar *last-command* nil
   "Set to the last interactive command run.")
@@ -932,6 +965,11 @@ raise/map denial messages will be seen.")
                 deny-list)
        t)))
 
+(defun flatten (list)
+  "Flatten LIST"
+  (labels ( (mklist (x) (if (listp x) x (list x))) )
+    (mapcan #'(lambda (x) (if (atom x) (mklist x) (flatten x))) list)))
+
 (defun list-splice-replace (item list &rest replacements)
   "splice REPLACEMENTS into LIST where ITEM is, removing
 ITEM. Return the new list."
@@ -1075,7 +1113,7 @@ input focus is transfered to the window you click on.")
 
 (defvar *root-click-focuses-frame* t
   "Set to NIL if you don't want clicking the root window to focus the frame
-  containing the pointer when *mouse-focus-policy* is :click.")
+  containing the pointer.")
 
 (defvar *banish-pointer-to* :head
   "Where to put the pointer when no argument is given to (banish-pointer) or the banish
@@ -1145,7 +1183,7 @@ of :error."
   (declare (ignorable if-exists))
   `(progn
      (ensure-directories-exist *data-dir*)
-     (with-open-file (,s ,(merge-pathnames *data-dir* file)
+     (with-open-file (,s ,(merge-pathnames file *data-dir*)
                          ,@keys)
        ,@body)))
 
