@@ -27,6 +27,11 @@
 (export '(current-screen
           current-window
           screen-current-window
+	  screen-number
+	  screen-groups
+	  screen-windows
+	  screen-height
+	  screen-width
           set-fg-color
           set-bg-color
           set-border-color
@@ -224,7 +229,8 @@ identity with a range check."
 
 (defun update-border-for-screen (screen)
   (setf (xlib:drawable-border-width (screen-input-window screen)) (screen-msg-border-width screen)
-        (xlib:drawable-border-width (screen-message-window screen)) (screen-msg-border-width screen)))
+        (xlib:drawable-border-width (screen-message-window screen)) (screen-msg-border-width screen)
+        (xlib:drawable-border-width (screen-frame-window screen)) (screen-msg-border-width screen)))
 
 (defun update-border-all-screens ()
   "After setting the border width call this to sync any existing windows."
@@ -243,9 +249,9 @@ identity with a range check."
             always (xlib:lookup-color (xlib:screen-default-colormap (screen-number i)) color))
     (xlib:name-error () nil)))
 
-(defun font-exists-p (font-name)
-  ;; if we can list the font then it exists
-  (plusp (length (xlib:list-font-names *display* font-name :max-fonts 1))))
+;; (defun font-exists-p (font-name)
+;;   ;; if we can list the font then it exists
+;;   (plusp (length (xlib:list-font-names *display* font-name :max-fonts 1))))
 
 (defmacro set-any-color (val color)
   `(progn (dolist (s *screen-list*)
@@ -297,8 +303,7 @@ there is more than one frame."
   (set-any-color screen-float-unfocus-color color))
 
 (defun set-msg-border-width (width)
-  "Set the border width for the message bar and input
-bar."
+  "Set the border width for the message bar, input bar and frame indicator."
   (check-type width (integer 0))
   (dolist (i *screen-list*)
     (setf (screen-msg-border-width i) width))
@@ -314,20 +319,17 @@ bar."
   t)
 
 (defun set-font (font)
-  "Set the font for the message bar and input bar."
-  (when (font-exists-p font)
-    (dolist (i *screen-list*)
-      (let ((fobj (xlib:open-font *display* (first (xlib:list-font-names *display* font :max-fonts 1)))))
-        (xlib:close-font (screen-font i))
-        (setf (screen-font i) fobj
-              (xlib:gcontext-font (screen-message-gc i)) fobj)
-        ;; update the modelines too
-        (dolist (h (screen-heads i))
-          (when (and (head-mode-line h)
-                     (eq (mode-line-mode (head-mode-line h)) :stump))
-            (setf (xlib:gcontext-font (mode-line-gc (head-mode-line h))) fobj)
-            (resize-mode-line (head-mode-line h))
-            (sync-mode-line (head-mode-line h))))))
+  "Set the font(s) for the message bar and input bar."
+  (when (if (listp font)
+            (every #'identity (mapcar #'font-exists-p font))
+            (font-exists-p font))
+    (dolist (screen *screen-list*)
+      (let ((fonts (if (listp font)
+                       (mapcar (lambda (font) (open-font *display* font))
+                               font)
+                       (list (open-font *display* font)))))
+        (mapc #'close-font (screen-fonts screen))
+        (setf (screen-fonts screen) fonts)))
     t))
 
 (defmacro with-current-screen (screen &body body)
@@ -434,10 +436,10 @@ FOCUS-WINDOW is an extra window used for _NET_SUPPORTING_WM_CHECK."
                                              :colormap (xlib:screen-default-colormap
                                                         screen-number)
                                              :event-mask '(:exposure)))
-           (font (xlib:open-font *display*
-                                 (if (font-exists-p +default-font-name+)
-                                     +default-font-name+
-                                     "*")))
+           (font (open-font *display*
+                            (if (font-exists-p +default-font-name+)
+                                +default-font-name+
+                                "*")))
            (group (make-instance 'tile-group
                    :screen screen
                    :number 1
@@ -451,7 +453,7 @@ FOCUS-WINDOW is an extra window used for _NET_SUPPORTING_WM_CHECK."
             (screen-host screen) host
             (screen-groups screen) (list group)
             (screen-current-group screen) group
-            (screen-font screen) font
+            (screen-fonts screen) (list font)
             (screen-fg-color screen) fg
             (screen-bg-color screen) bg
             (screen-win-bg-color screen) win-bg
@@ -468,13 +470,15 @@ FOCUS-WINDOW is an extra window used for _NET_SUPPORTING_WM_CHECK."
             (screen-frame-window screen) frame-window
             (screen-ignore-msg-expose screen) 0
             (screen-message-cc screen) (make-ccontext :win message-window
+                                                      :screen screen
+                                                      :font font
                                                       :gc (xlib:create-gcontext
                                                            :drawable message-window
-                                                           :font font
+                                                           :font (when (typep font 'xlib:font) font)
                                                            :foreground fg
                                                            :background bg))
             (screen-frame-outline-gc screen) (xlib:create-gcontext :drawable (screen-root screen)
-                                                                   :font font
+                                                                   :font (when (typep font 'xlib:font) font)
                                                                    :foreground fg
                                                                    :background fg
                                                                    :line-style :double-dash
@@ -485,7 +489,7 @@ FOCUS-WINDOW is an extra window used for _NET_SUPPORTING_WM_CHECK."
       (netwm-set-properties screen focus-window)
       (update-colors-for-screen screen)
       (update-color-map screen)
-      (xwin-grab-keys focus-window screen)
+      (xwin-grab-keys focus-window group)
       screen)))
 
 ;;; Screen commands
