@@ -29,6 +29,7 @@
 	  programs-in-path
 	  restarts-menu
 	  run-or-raise
+          run-or-pull
 	  run-shell-command
           window-send-string))
 
@@ -174,9 +175,11 @@ such a case, kill the shell command to resume StumpWM."
 (defcommand eval-line (cmd) ((:rest "Eval: "))
   "Evaluate the s-expression and display the result(s)."
   (handler-case
-      (message "^20~{~a~^~%~}"
+      (if cmd
+          (message "^20~{~a~^~%~}"
                (mapcar 'prin1-to-string
                        (multiple-value-list (eval (read-from-string cmd)))))
+          (throw 'error :abort))
     (error (c)
       (err "^B^1*~A" c))))
 
@@ -257,7 +260,8 @@ number, with group being more significant (think radix sort)."
     (stable-sort (sort matches #'< :key #'window-number)
                  #'< :key (lambda (w) (group-number (window-group w))))))
 
-(defun run-or-raise (cmd props &optional (all-groups *run-or-raise-all-groups*) (all-screens *run-or-raise-all-screens*))
+(defun run-or-raise (cmd props &optional (all-groups *run-or-raise-all-groups*)
+                                 (all-screens *run-or-raise-all-screens*))
   "Run the shell command, @var{cmd}, unless an existing window
 matches @var{props}. @var{props} is a property list with the following keys:
 
@@ -283,7 +287,6 @@ instance. @var{all-groups} overrides this default. Similarily for
          (let* ((group (window-group win))
                 (frame (window-frame win))
                 (old-frame (tile-group-current-frame group)))
-           (frame-raise-window group frame win)
            (focus-all win)
            (unless (eq frame old-frame)
              (show-frame-indicator group)))))
@@ -297,7 +300,7 @@ instance. @var{all-groups} overrides this default. Similarily for
                     (first matches))))
       (if win
           (if (eq (type-of (window-group win)) 'float-group)
-              (group-focus-window (window-group win) win)
+              (focus-all win)
               (goto-win win))
           (run-shell-command cmd)))))
 
